#!/usr/bin/env python3
# gcalendar is a tool to read Google Calendar events from your terminal.

# Copyright (C) 2020  Gobinath

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import socket
from datetime import datetime
from os.path import join

from googleapiclient import discovery
from httplib2 import Http
from oauth2client import client
from oauth2client import file
from oauth2client import tools

from gcalendar import GOOGLE_CALENDAR_SCOPE, AUTH_URI, TOKEN_URI


class GCalendar:
    def __init__(self, client_id, client_secret, account_id, config_dir):
        super().__init__()
        self.client_id = client_id
        self.client_secret = client_secret
        self.account_id = account_id
        self.storage = join(config_dir, account_id + "_v1.dat")
        self.service = self.create_service()

    def create_service(self):
        # Prepare credentials, and authorize HTTP object with them.
        # If the credentials don"t exist or are invalid run through the native client
        # flow. The Storage object will ensure that if successful the good
        # credentials will get written back to a file.

        # Create a unique storage for the given account_id
        storage = file.Storage(self.storage)
        credentials = storage.get()

        # If credentials not found, authenticate
        if credentials is None or credentials.invalid:
            # Set up a Flow object to be used if we need to authenticate.
            flow = self.__flow_from_client_secrets()
            credentials = tools.run_flow(flow, storage, DefaultArg())

        http = credentials.authorize(http=Http())

        # Construct a service object via the discovery service.
        return discovery.build("calendar", "v3", http=http)

    def list_calendars(self):
        """List calendar names"""
        calendars = list()
        page_token = None
        while True:
            calendar_list = self.service.calendarList().list(pageToken=page_token).execute()
            for calendar_list_entry in calendar_list["items"]:
                calendars.append(calendar_list_entry["summary"])
            page_token = calendar_list.get("nextPageToken")
            if not page_token:
                break
        return calendars

    def list_events(self, calendars, start_time, end_time, time_zone):
        """List sorted calendar events"""
        calendar_events = list()
        all_calendars = "*" in calendars
        page_token = None
        while True:
            calendar_list = self.service.calendarList().list(pageToken=page_token).execute()
            for calendar_list_entry in calendar_list["items"]:
                if all_calendars or (calendar_list_entry["summary"].lower() in calendars):
                    events = self.retrieve_events(calendar_list_entry["id"], calendar_list_entry["backgroundColor"],
                                                  start_time, end_time, time_zone)
                    calendar_events.extend(events)
            page_token = calendar_list.get("nextPageToken")
            if not page_token:
                break

        return sorted(calendar_events, key=lambda event: event["start_date"] + event["start_time"])

    def retrieve_events(self, calendar_id, calendar_color, start_time, end_time, time_zone):
        page_token = None
        retrieved_events = []
        while True:
            events = self.service.events().list(calendarId=calendar_id,
                                                pageToken=page_token,
                                                timeMin=start_time,
                                                timeMax=end_time,
                                                timeZone=time_zone,
                                                singleEvents=True).execute()
            for event in events["items"]:
                calendar_event = {"calendar_color": calendar_color, "summary": event.get("summary", "NO_TITLE")}
                # Extract the start and end time
                if "dateTime" in event["start"]:
                    start_date_time = datetime.strptime(
                        "".join(event["start"]["dateTime"].rsplit(":", 1)), "%Y-%m-%dT%H:%M:%S%z")
                    end_date_time = datetime.strptime(
                        "".join(event["end"]["dateTime"].rsplit(":", 1)), "%Y-%m-%dT%H:%M:%S%z")
                    calendar_event["start_date"] = str(start_date_time.date())
                    calendar_event["start_time"] = str(start_date_time.time()).rsplit(":", 1)[0]
                    calendar_event["end_date"] = str(end_date_time.date())
                    calendar_event["end_time"] = str(end_date_time.time()).rsplit(":", 1)[0]
                else:
                    calendar_event["start_date"] = event["start"]["date"]
                    calendar_event["start_time"] = "00:00"
                    calendar_event["end_date"] = event["end"]["date"]
                    calendar_event["end_time"] = "00:00"

                # Extract the location
                if "location" in event:
                    calendar_event["location"] = event["location"]
                else:
                    calendar_event["location"] = ""
                retrieved_events.append(calendar_event)
            page_token = events.get("nextPageToken")
            if not page_token:
                break
        return retrieved_events

    def __flow_from_client_secrets(self):
        constructor_kwargs = {
            "redirect_uri": None,
            "auth_uri": AUTH_URI,
            "token_uri": TOKEN_URI,
            "login_hint": None,
        }
        return client.OAuth2WebServerFlow(self.client_id, self.client_secret,
                                          GOOGLE_CALENDAR_SCOPE, **constructor_kwargs)


class DefaultArg:

    def __init__(self):
        self.auth_host_name = "localhost"
        self.noauth_local_webserver = False
        self.auth_host_port = DefaultArg.get_available_ports()
        self.logging_level = "ERROR"

    @staticmethod
    def get_available_ports():
        soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        soc.bind(("", 0))
        port = soc.getsockname()[1]
        soc.close()
        return [port, 8081, 8788, 8091, 8098]



import os

import setuptools

requires = [
    'python-dateutil',
    'google-api-python-client',
    'oauth2client',
    'httplib2'
]

_ROOT = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(_ROOT, 'README.md')) as f:
    long_description = f.read()


def __package_files(directory):
    """
    Collect the package files.
    """
    paths = []
    for (path, _, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join('..', path, filename))
    return paths


setuptools.setup(
    name="gcalendar",
    version="0.1",
    description="Read Google Calendar events from your terminal.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Gobinath Loganathan",
    author_email="slgobinath@gmail.com",
    url="https://github.com/slgobinath/gcalendar",
    download_url="https://github.com/slgobinath/gcalendar/archive/v0.1.tar.gz",
    packages=setuptools.find_packages(),
    package_data={},
    install_requires=requires,
    entry_points={'console_scripts': ['gcalendar = gcalendar.__main__:main']},
    keywords='linux utility google-calendar',
    classifiers=[
                    "Operating System :: POSIX :: Linux",
                    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
                    "Development Status :: 5 - Production/Stable",
                    "Environment :: X11 Applications :: GTK",
                    "Intended Audience :: End Users/Desktop",
                    "Topic :: Utilities"] + [('Programming Language :: Python :: %s' % x) for x in
                                             '3 3.4 3.5 3.6 3.7'.split()]
)



#!/usr/bin/env python3
# gcalendar is a tool to read Google Calendar events from your terminal.

# Copyright (C) 2020  Gobinath

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import json
import os
import sys
from datetime import datetime, timezone

from dateutil.relativedelta import relativedelta
from oauth2client import client
from oauth2client import clientsecrets

from gcalendar import DEFAULT_CLIENT_ID, DEFAULT_CLIENT_SECRET
from gcalendar.gcalendar import GCalendar

# the home folder
HOME_DIRECTORY = os.environ.get('HOME') or os.path.expanduser('~')

# ~/.config/gcalendar folder
CONFIG_DIRECTORY = os.path.join(os.environ.get(
    'XDG_CONFIG_HOME') or os.path.join(HOME_DIRECTORY, '.config'), 'gcalendar')


def validate_account_id(account_id):
    """
    Validate the argparse argument --account
    """
    account = str(account_id)
    if not account.isalnum():
        raise argparse.ArgumentTypeError(
            "%s is not an alphanumeric id" % account)
    return account


def error(message, current_time):
    print(
        '[{"calendar_color": "#ffffff", "summary": "' + message +
        '", "start_date": "%s", "start_time": "00:00", "end_date": "%s", "end_time": "00:00", "location": ""}]' % (
            current_time.date(), (current_time + relativedelta(days=1)).date()))


def main(argv):
    """
    Retrieve Google Calendar events.
    """
    parser = argparse.ArgumentParser(
        description="Retrieve Google Calendar events.")
    parser.add_argument("--no-of-days", type=str, default="7",
                        help="number of days to include")
    parser.add_argument("--calendar", type=str, default=["*"], nargs="*")
    parser.add_argument("--list-calendars", action="store_true")
    parser.add_argument("--client-id", type=str, help="the Google client id")
    parser.add_argument("--client-secret", type=str,
                        help="the Google client secret")
    parser.add_argument("--account", type=validate_account_id, default="default",
                        help="a name to uniquely identify the account")
    args = parser.parse_args()

    # Extract arguments
    no_of_days = int(args.no_of_days)
    client_id = args.client_id
    client_secret = args.client_secret
    selected_calendars = [x.lower() for x in args.calendar]
    account_id = args.account

    current_time = datetime.now(timezone.utc).astimezone()
    time_zone = str(current_time.tzinfo)
    start_time = str(current_time.isoformat())
    end_time = str((current_time + relativedelta(days=no_of_days)).isoformat())

    if not client_id or not client_secret:
        client_id = DEFAULT_CLIENT_ID
        client_secret = DEFAULT_CLIENT_SECRET

    # Create the config folder if not exists
    if not os.path.exists(CONFIG_DIRECTORY):
        os.mkdir(CONFIG_DIRECTORY)

    try:
        g_calendar = GCalendar(client_id, client_secret, account_id, CONFIG_DIRECTORY)
        if args.list_calendars:
            for calendar in g_calendar.list_calendars():
                print(calendar)
        else:
            calendar_events = g_calendar.list_events(selected_calendars, start_time, end_time, time_zone)
            if calendar_events:
                print(json.dumps(calendar_events))
            else:
                error("NO_EVENTS_FOUND_GOOGLE_CALENDAR", current_time)

    except clientsecrets.InvalidClientSecretsError:
        error("INVALID_CLIENT_SECRETS", current_time)
    except client.AccessTokenRefreshError:
        error("FAILED_TO_REFRESH_ACCESS_TOKEN", current_time)
    except BaseException:
        error("FAILED_TO_RETRIEVE_EVENTS", current_time)


if __name__ == "__main__":
    main(sys.argv)



#!/usr/bin/env python3
# gcalendar is a tool to read Google Calendar events from your terminal.

# Copyright (C) 2020  Gobinath

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

GOOGLE_CALENDAR_SCOPE = "https://www.googleapis.com/auth/calendar.readonly"

AUTH_URI = "https://accounts.google.com/o/oauth2/auth"

TOKEN_URI = "https://accounts.google.com/o/oauth2/token"

DEFAULT_CLIENT_ID = "1097830982036-5opi3qc4lje5ak0so1mjg0icktiiu289.apps.googleusercontent.com"

DEFAULT_CLIENT_SECRET = "S5zEdU60nQfdaS3AXXTLNivB"
